// netstat.c - NAT32 Network Status Application
// Version 0.2
// 5.3.2020
// Copyright NAT Software, Australia
// NOTE: Add #include <winres.h> to top of netstat.rc

#define _CRT_SECURE_NO_WARNINGS

#include <windows.h>
#include <winsock.h>
#include <stdio.h>
#include "sockio.h"
#include "netstat.h"

#pragma comment(lib, "wsock32.lib")
#pragma comment(lib, "powrprof.lib")

BOOLEAN WINAPI SetSuspendState(BOOLEAN, BOOLEAN, BOOLEAN);

HANDLE hThread;
HANDLE hSem;
HINSTANCE hInst;
HWND ghDlg;
HWND hCtrlEdit;
HWND hCtrlList;

NOTIFYICONDATA iconData;

#define MYWM_NOTIFYICON 200

int icon_state;
int active;
int icheck;

#define BAD	   -1
#define NONE	0
#define MAIN	1
#define ALT		2
#define BOTH	3


int  flag, tid;
char gateway[256];
char status_txt[256];

// Function prototypes

char *state2str(char *buf, char *gateway, int state, int active);
BOOL APIENTRY NetStatDlgProc (HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam);
BOOL APIENTRY HelpDlgProc (HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam);
int parse(LPSTR CmdLine);
int exec(char *cmdline, short flag);
BOOL CenterWindow(HWND hWin);

//
// kprintf - low-level printf
//
int kprintf(char *fmt, ...)
{
    va_list args;
    char buf[0x8000];

    va_start(args, fmt);
    vsprintf(buf, fmt, args);
    va_end(args);

    return MessageBox(NULL, buf, "NETSTAT", MB_OKCANCEL | MB_SETFOREGROUND);
}

//
// dprintf - low-level printf via OutputDebugString
//           Run www.sysinternals.com DebugView.exe
//           to view this output.
//

int dprintf(char *fmt, ...)
{
    va_list args;
    char buf[0x8000];
    int n;

    va_start(args, fmt);

    n = vsprintf(buf, fmt, args);

    va_end(args);

    if (n < 0)
        return SYSERR;

    buf[n] = 0;

    OutputDebugString(buf);

    return n;
}

/*------------------------------------------------------------------------
 *
 *  Status thread function
 *
 *------------------------------------------------------------------------
*/

DWORD WINAPI status(void *param) {

    int len, s, wait_time;
    char buf[1500], tmp[256];

    while (TRUE) {		// This thread never terminates

        icon_state = BAD;

		// Use a new socket each time, or sReadT won't work

        s = connectUDP(gateway, "echo");

		if (s == SYSERR) {
			
			// The gateway name didn't resolve

			goto done;
		}

        sWrite(s, "status", 7);

        len = sReadT(s, buf, 1400, 5000);		// 5 sec (the 2 checki's might timeout)

        // If we got an ICMP response, retry the sReadT

        if (len == -1)
            len = sReadT(s, buf, 1400, 5000);	// 5 sec timeout

        if (len == -3) {

            // Server Unreachable

			CheckDlgButton(ghDlg, IDC_CHECK1, BST_INDETERMINATE); 
            sprintf(status_txt, "No response from %s", gateway);
            SetDlgItemText(ghDlg, IDC_EDIT2, status_txt);
            wait_time = 10000;                  // Slow 10 + 5 sec poll
            goto done;
        }
		else
			wait_time = 5000;					// Fast 5 sec poll

		if (buf[0] == '0') {

            // Main gateway is down

	        icon_state = NONE;
        }

        if (buf[0] == '1') {

            // Main gateway is up

            icon_state = MAIN;
        }

        if (buf[2] == '2') {

            // Alternate gateway is up

			icon_state += ALT;

		}

		sscanf(&buf[4], "%c", &active);
		sscanf(&buf[6], "%c", &icheck);
		CheckDlgButton(ghDlg, IDC_CHECK1, atoi((char *)&icheck)); 

done:	switch (icon_state) {
			case BAD:	iconData.hIcon = LoadIcon(hInst, "ICON2_BAD");
						break;
			case NONE:	iconData.hIcon = LoadIcon(hInst, "ICON3_NONE");
						break;
			case MAIN:	iconData.hIcon = LoadIcon(hInst, "ICON4_MAIN");
						break;
			case ALT:	iconData.hIcon = LoadIcon(hInst, "ICON5_ALT");
						break;
			case BOTH:	iconData.hIcon = LoadIcon(hInst, "ICON6_BOTH");
						break;
			default:	iconData.hIcon = LoadIcon(hInst, "ICON2_BAD");
		}
		
		state2str(status_txt, gateway, icon_state, active);
		GetDlgItemText(ghDlg, IDC_EDIT2, tmp, 256);
		if (strcmp(tmp, status_txt))
			SetDlgItemText(ghDlg, IDC_EDIT2, status_txt);
		strcpy(iconData.szTip, status_txt);
		Shell_NotifyIcon(NIM_MODIFY, &iconData);
		
		// This changes the application's icon

		SendMessage(ghDlg, WM_SETICON, ICON_SMALL, (LPARAM)iconData.hIcon);

		// Check socket in case the gateway name didn't resolve

		if (s == SYSERR) {
			flag = 1;
			CheckDlgButton(ghDlg, IDC_CHECK1, BST_INDETERMINATE); 
            sprintf(status_txt, "Gateway %s is invalid", gateway);
            SetDlgItemText(ghDlg, IDC_EDIT2, status_txt);
            ShowWindow(ghDlg, SW_RESTORE);
			wait_time = 10000;	// 10 seconds
		}
		else
			sClose(s);
		
		// If the alternate gateway is down, always switch to DSL

		if ((icon_state & ALT) == 0)
			SendMessage(ghDlg, WM_COMMAND, IDC_BUTTON1, 0);
        
		WaitForSingleObject(hSem, wait_time);
    }
    return 0;
}

int WINAPI WinMain(HINSTANCE hExe, HINSTANCE hPrev, LPSTR CmdLine, int CmdShow)
{
    WSADATA data;
    int result;
    WNDCLASS wc;
	HWND hWnd = FindWindow("NetStatDlgClass", NULL);

    int status;


	if (hWnd) {
		kprintf("ERROR: NETSTAT is already running.\n");
        return OK;
	}

    hInst = hExe;

    status = WSAStartup(MAKEWORD(1,1), &data);
    if (status) {
        kprintf("WINSOCK startup error");
        return 0;
    }

	hSem = CreateSemaphore(NULL, 0, 1, 0);

	if (strlen(CmdLine) == 0)
		strcpy(gateway, "nat32.box");
	else
		strcpy(gateway, CmdLine);

    // Create the dialog box

    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = DefDlgProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = DLGWINDOWEXTRA;
    wc.hInstance = hInst;
    wc.hIcon = LoadIcon(hInst, "ICON1_NETSTAT");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH) COLOR_WINDOW + 1;
    wc.lpszMenuName = NULL;
    wc.lpszClassName = "NetStatDlgClass";
    RegisterClass(&wc);

    result = DialogBox(hInst, "NETSTAT", NULL, NetStatDlgProc);
    if (result)
        kprintf("FATAL ERROR [%d]: unable to create NETSTAT dialog\n", GetLastError());

    if (hThread)
        CloseHandle(hThread);

    Shell_NotifyIcon(NIM_DELETE, &iconData);

    status = WSACleanup();
    if (status)
        kprintf("WINSOCK cleanup error");

    return 0;
}

//
// NetStatDlgProc - all the work is done here
//

BOOL APIENTRY NetStatDlgProc (HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
    int  len, s, param, wmID;
    char buf[1500];

	switch (msg) {

        case WM_INITDIALOG:

            ghDlg = hDlg;

            hCtrlEdit = GetDlgItem(hDlg, IDC_EDIT1);
            hCtrlList = GetDlgItem(hDlg, IDC_LISTBOX1);

            SendDlgItemMessage(hDlg, IDC_LISTBOX1, LB_SETHORIZONTALEXTENT, (WPARAM) 128*8 /* pixels, max 32767 */, (LPARAM) 0);

			icon_state = NONE;

            iconData.cbSize = sizeof(iconData);
            iconData.hWnd = hDlg;
            iconData.uID = 1;   // any old value
            iconData.uFlags = NIF_ICON | NIF_TIP | NIF_MESSAGE;
            iconData.uCallbackMessage = MYWM_NOTIFYICON;
            iconData.hIcon = LoadIcon(hInst, "ICON2_BAD");
            strcpy(status_txt, "Internet Status");
            strcpy(iconData.szTip, status_txt);
            Shell_NotifyIcon(NIM_ADD, &iconData);

            CenterWindow(hDlg);

			CheckDlgButton(hDlg, IDC_CHECK1, BST_INDETERMINATE); 
            
			SetDlgItemText(hDlg, IDC_EDIT1, gateway);
            PostMessage(hDlg, WM_COMMAND, IDOK, 0);

			return TRUE;

        case WM_DESTROY:

            PostQuitMessage(0);
            break;

        case WM_CLOSE:

            flag = 0;
            ShowWindow(hDlg, SW_HIDE);
            return TRUE;

        case MYWM_NOTIFYICON:

			state2str(status_txt, gateway, icon_state, active);
			strcpy(iconData.szTip, status_txt);
            Shell_NotifyIcon(NIM_MODIFY, &iconData);
			SetDlgItemText(hDlg, IDC_EDIT2, status_txt);

            param = (UINT) lParam;

            if (param == WM_LBUTTONDBLCLK) {
                break;
            }

            if (param == WM_RBUTTONUP) {
                if (flag == 0) {
                    SetForegroundWindow(hDlg);
                    ShowWindow(hDlg, SW_RESTORE);
                    flag = 1;
                }
                else {
                    ShowWindow(hDlg, SW_HIDE);
                    flag = 0;
                }
                return TRUE;
            }

            if (param == WM_LBUTTONDOWN) {
                if (flag == 0) {
                    SetForegroundWindow(hDlg);
                    ShowWindow(hDlg, SW_RESTORE);
                    flag = 1;
                }
                else {
                    ShowWindow(hDlg, SW_HIDE);
                    flag = 0;
                }
            }
            return TRUE;

        case WM_COMMAND:

            wmID = LOWORD(wParam);

            switch (wmID) {

                case IDOK:

                    len = GetDlgItemText(hDlg, IDC_EDIT1, gateway, 256);

                    if (len >= 1) {
						if (hThread == 0) {
                            hThread = CreateThread(0, 0, status, 0, 0, &tid);
	                        sprintf(status_txt, "Internet status for %s", gateway);
		                    strcpy(iconData.szTip, status_txt);
			                Shell_NotifyIcon(NIM_MODIFY, &iconData);
						}
                        else {
                            ReleaseSemaphore(hSem, 1, 0);
                        }
                    }

                    if (hThread) {
                        ShowWindow(hDlg, SW_HIDE);
                        flag = 0;
                        return TRUE;
                    }
                    SetDlgItemText(hDlg, IDC_EDIT2, "No gateway specified");
                    SetFocus(hCtrlEdit);
                    return FALSE;

                case IDCANCEL:
                    EndDialog(hDlg, IDOK);
                    return TRUE;

				case IDC_BUTTON1:
			        s = connectUDP(gateway, "echo");
					sWrite(s, "gateway1", 9);
		            len = sReadT(s, buf, 1400, 5000);	// 5 sec timeout
					if (len > 0) {
						CheckDlgButton(hDlg, IDC_CHECK1, BST_UNCHECKED);
	                    SetDlgItemText(hDlg, IDC_EDIT2, "Main gateway set");
					}
					sClose(s);
					return FALSE;

				case IDC_BUTTON2:
			        s = connectUDP(gateway, "echo");
					sWrite(s, "gateway2", 9);
		            len = sReadT(s, buf, 1400, 5000);	// 5 sec timeout
					if (len > 0) {
						CheckDlgButton(hDlg, IDC_CHECK1, BST_UNCHECKED);
	                    SetDlgItemText(hDlg, IDC_EDIT2, "Alternate gateway set");
					}
					sClose(s);
					return FALSE;

				case IDC_BUTTON3:
					SendDlgItemMessage(hDlg, IDC_LISTBOX1, LB_RESETCONTENT, 0, 0);
			        s = connectUDP(gateway, "echo");
					sWrite(s, "log", 4);
					while (1) {
						len = sReadT(s, buf, 1400, 5000);	// 5 sec timeout
						if (len > 1)
							SendDlgItemMessage(hDlg, IDC_LISTBOX1, LB_ADDSTRING, 0, (LPARAM) buf);
						else
							break;
					}
					SendDlgItemMessage(hDlg, IDC_LISTBOX1, WM_VSCROLL, SB_BOTTOM, 0);  
					sClose(s);
					return FALSE;

				case IDC_BUTTON4:
					SetFocus(0);	// Ignore keyboard input
					if (SetSuspendState(0, 0, 0))
						dprintf("Entering STANDBY state NOW\n");
					return FALSE;

				case IDC_CHECK1:
			        s = connectUDP(gateway, "echo");
					SetDlgItemText(hDlg, IDC_EDIT2, "Changing mode ...");
					if (IsDlgButtonChecked(hDlg, IDC_CHECK1)) {
						CheckDlgButton(hDlg, IDC_CHECK1, BST_UNCHECKED); 
						sWrite(s, "clr icheck", 10);
					}
					else {
						CheckDlgButton(hDlg, IDC_CHECK1, BST_CHECKED); 
						sWrite(s, "set icheck", 10);
					}
		            len = sReadT(s, buf, 1400, 5000);	// 5 sec timeout
					if (len > 0) {
						buf[len] = 0;
	                    SetDlgItemText(hDlg, IDC_EDIT2, buf);
					}
					else {
						CheckDlgButton(hDlg, IDC_CHECK1, BST_INDETERMINATE); 
	                    SetDlgItemText(hDlg, IDC_EDIT2, "Gateway did not respond");
					}

					sClose(s);
					return FALSE;
            }
        break;
    }
    return FALSE;
}

char *state2str(char *buf, char *gateway, int state, int active) {

	switch (state) {
		case BAD:   sprintf(buf, "No response from %s", gateway);
					break;

		case NONE:  strcpy(buf, "No Internet connection");
					break;

		case MAIN:	strcpy(buf, "Main Internet");
					if (active == '1')
						strcat(buf, " active");
					break;

		case ALT:	strcpy(buf, "Alternate Internet");
					if (active == '2')
						strcat(buf, " active");
					break;

		case BOTH:	switch (active) {
					case '1': strcpy(buf, "Main connection active");
							  break;
					case '2': strcpy(buf, "Alternate connection active");
							  break;
					default:  strcpy(buf, "Both Internet connections OK");
					}
					break;
		default:   strcpy(buf, "No Internet connection");
	}
	return buf;
}

BOOL CenterWindow(HWND hWin)
{
    int result;
    HWND hDesktop;

    RECT rc;
    RECT rcd;

    hDesktop = GetDesktopWindow();
    result = GetWindowRect(hWin, &rc);

    if (result)
        result = GetWindowRect(hDesktop, &rcd);

    if (result == 0)
        return -1;

    SetWindowPos(hWin, HWND_TOP,
                 rcd.right/2 - (rc.right - rc.left)/2,
                 rcd.bottom/2 - (rc.bottom - rc.top)/2,
                 0, 0,
                 SWP_NOSIZE);
    return 0;
}

/* sockio - connectUDP connectTCP passiveUDP passiveTCP
            sRead sWrite sReadFrom sWriteTo sClose sShutdown
            wname2ip wip2name 
*/

u_short portbase = 0;        /* port base, for non-root servers */

/*------------------------------------------------------------------------
 * connectUDP - connect to a specified UDP service on a specified host
 *------------------------------------------------------------------------
*/
int
connectUDP( host, service )
char	*host;		/* name of host to which connection is desired	*/
char	*service;	/* service associated with the desired port	*/
{
    return connectsock(host, service, "udp");
}

/*------------------------------------------------------------------------
 * connectTCP - connect to a specified TCP service on a specified host
 *------------------------------------------------------------------------
*/
int 
connectTCP( host, service )
char	*host;		/* name of host to which connection is desired	*/
char	*service;	/* service associated with the desired port	*/
{
	return connectsock( host, service, "tcp");
}
/*------------------------------------------------------------------------
 * passiveUDP - create a passive socket for use in a UDP server
 *------------------------------------------------------------------------
*/
int 
passiveUDP( service )
char    *service;           /* service associated with the desired port */
{
    return passivesock(service, "udp", 0);
}

/*------------------------------------------------------------------------
 * passiveTCP - create a passive socket for use in a TCP server
 *------------------------------------------------------------------------
*/
int 
passiveTCP( service, qlen )
char    *service;           /* service associated with the desired port */
int qlen;                   /* maximum server request queue length      */
{
	return passivesock(service, "tcp", qlen);
}

/*------------------------------------------------------------------------
 * sAccept - accept a TCP connection
 *------------------------------------------------------------------------
*/
int 
sAccept( s, saddr)
SOCKET s;
struct saddr *saddr;
{
    int alen = sizeof(struct sockaddr);
    return accept(s, (struct sockaddr *) saddr, &alen);
}

/*------------------------------------------------------------------------
 *  sRead - receive from a connected socket
 *------------------------------------------------------------------------
*/
int 
sRead(s, buf, len)
SOCKET s;
char *buf;
int len;
{
    return recv(s, buf, len, 0);
}

/*------------------------------------------------------------------------
 *  sReadT - receive from a connected socket with TIMEOUT in msec
 *------------------------------------------------------------------------
*/
int 
sReadT(s, buf, len, time)
SOCKET s;
char *buf;
int len;
int time;
{
    struct timeval timeout;
    int    result;
    fd_set rfds;

    timeout.tv_sec = 0;
    timeout.tv_usec = time * 1000;


    FD_ZERO(&rfds);
    FD_SET(s, &rfds);

    result = select(FD_SETSIZE, &rfds, (fd_set *) 0, (fd_set *) 0, &timeout);

    switch (result) {

        case 0: return TIMEOUT;

        case 1: return recv(s, buf, len, 0);

        case SOCKET_ERROR: dprintf("sReadT: select returned SOCKET_ERROR %d\n", WSAGetLastError());
                           return SYSERR;

        default: return SYSERR;
    }
}
/*------------------------------------------------------------------------
 *  sReadFrom - receive a datagram from a connected or unconnected socket
 *------------------------------------------------------------------------
*/
int 
sReadFrom(s, buf, len, saddr)
SOCKET s;
char *buf;
int len;
struct saddr *saddr;
{
    int alen = sizeof(struct saddr);

    saddr->sin_family = AF_INET;
    return recvfrom(s, buf, len, 0, (struct sockaddr *) saddr, &alen);
}

/*------------------------------------------------------------------------
 *  sReadFromT - receive a datagram with timeout from (un)connected socket
 *------------------------------------------------------------------------
*/
int 
sReadFromT(s, buf, len, saddr, time)
SOCKET s;
char *buf;
int len;
struct saddr *saddr;
int time;
{
    struct timeval timeout;
    int    result;
    fd_set rfds;
    int alen = sizeof(struct saddr);

    timeout.tv_sec = 0;
    timeout.tv_usec = time * 1000;

    FD_ZERO(&rfds);
    FD_SET(s, &rfds);

    result = select(FD_SETSIZE, &rfds, (fd_set *) 0, (fd_set *) 0, &timeout);

    switch (result) {

        case 0: return TIMEOUT;

        case 1: saddr->sin_family = AF_INET;
                return recvfrom(s, buf, len, 0, (struct sockaddr *) saddr, &alen);

        case SOCKET_ERROR: printf("sReadT: select returned SOCKET_ERROR\n");
                           return SYSERR;

        default: return SYSERR;
    }
}

/*------------------------------------------------------------------------
 *  sWrite - send to a connected socket
 *------------------------------------------------------------------------
*/
int 
sWrite(s, buf, len)
SOCKET s;
char *buf;
int len;
{
    return send(s, buf, len, 0);
}

/*------------------------------------------------------------------------
 *  sWriteTo - send to an unconnected socket
 *------------------------------------------------------------------------
*/
int 
sWriteTo(s, buf, len, saddr)
SOCKET s;
char *buf;
int len;
struct saddr *saddr;
{
    int alen = sizeof(struct saddr);
    saddr->sin_family = AF_INET;
    if (sendto(s, buf, len, 0, (struct sockaddr *) saddr, alen) <= 0)
        return h_errno;
    return OK;
//    return sendto(s, buf, len, 0, (struct sockaddr *) saddr, alen);
}

/*------------------------------------------------------------------------
 *  sClose  --  close a socket
 *------------------------------------------------------------------------
*/

int 
sClose(SOCKET s)
{

    if (s == SYSERR)
        return SYSERR;

    return closesocket(s);
}

/*------------------------------------------------------------------------
 *  sShutdown  --  shutdown a socket
 *------------------------------------------------------------------------
*/

int 
sShutdown(SOCKET s)
{
    int status;

    status = shutdown(s, 1);    /* disable sends */
    if (status)
        printf("\nsockio: shutdown failed\n");

    return status;
}

/*------------------------------------------------------------------------
 * sockalloc - allocate a UDP socket
 *------------------------------------------------------------------------
*/
int 
sockalloc()
{
    SOCKET s;

    if ((s = socket(PF_INET, SOCK_DGRAM, 0)) == INVALID_SOCKET) {
        printf("\nsockio: can't create socket: error %d\n", h_errno);
        return SYSERR;
    }
    return s;
}

/*------------------------------------------------------------------------
 * connectsock - allocate & connect a socket using TCP or UDP
 *------------------------------------------------------------------------
*/
int 
connectsock( host, service, protocol )
char	*host;		/* name of host to which connection is desired	*/
char	*service;	/* service associated with the desired port	*/
char	*protocol;	/* name of protocol to use ("tcp" or "udp")	*/
{
	struct hostent	*phe;	/* pointer to host information entry	*/
	struct servent	*pse;	/* pointer to service information entry	*/
	struct protoent *ppe;	/* pointer to protocol information entry*/
    struct sockaddr_in sin; /* an Internet endpoint address     */
    SOCKET s;               /* socket descriptor */
    int    type;            /* socket type    */
    char   option = TRUE;   /* enable the Boolean option */

    memset((char *)&sin, 0, sizeof(sin));
    sin.sin_family = AF_INET;

    /* Map service name to port number */

	if ( pse = getservbyname(service, protocol) )
		sin.sin_port = pse->s_port;
    else if ( (sin.sin_port = htons((u_short)atoi(service))) == 0 ) {
        printf("\nsockio: can't get %s service entry\n", service);
        return SYSERR;
    }

    /* Map host name to IP address, allowing for dotted decimal */

    /* CAUTION: inet_addr interprets an address such as 192.168.000.010
                as 192.168.0.8 and NOT 192.168.0.10
                In other words, it assumes that a leading zero indicates
                octal notation
                                                                      */

    if ( (sin.sin_addr.s_addr = inet_addr(host)) == INADDR_NONE )
        if ( phe = gethostbyname(host) )
            memcpy((char *)&sin.sin_addr, phe->h_addr, phe->h_length);
        else {
            printf("\nsockio: can't get %s host entry\n", host);
            return SYSERR;
        }

    /* Map protocol name to protocol number */

    if ( (ppe = getprotobyname(protocol)) == 0) {
        printf("\nsockio: can't get %s protocol entry\n", protocol);
        return SYSERR;
    }

    /* Use protocol to choose a socket type */

    if (strcmp(protocol, "udp") == 0)
        type = SOCK_DGRAM;
	else
        type = SOCK_STREAM;

    /* Allocate a socket */

    s = socket(PF_INET, type, ppe->p_proto);

    if (s == INVALID_SOCKET) {
        printf("\nsockio: can't create socket: error %d\n", h_errno);
        return SYSERR;
    }

    /* Connect the socket */

    if (connect(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
        printf("\nsockio: can't connect to %s:%s error %d\n", host, service, h_errno);
        return SYSERR;
    }

    /* Enable UDP broadcasts */

    if (type == SOCK_DGRAM)
        setsockopt(s, SOL_SOCKET, SO_BROADCAST, &option, sizeof(BOOL));
    return s;
}

/*------------------------------------------------------------------------
 * passivesock - allocate & bind a server socket using TCP or UDP
 *------------------------------------------------------------------------
 */
int 
passivesock( service, protocol, qlen )
char	*service;	/* service associated with the desired port	*/
char	*protocol;	/* name of protocol to use ("tcp" or "udp")	*/
int qlen;           /* maximum length of the server request queue   */
{
	struct servent	*pse;	/* pointer to service information entry	*/
	struct protoent *ppe;	/* pointer to protocol information entry*/
	struct sockaddr_in sin;	/* an Internet endpoint address		*/
    int s, type;            /* socket descriptor and socket type    */
    char   option = TRUE;   /* enable the Boolean option */

    memset((char *)&sin, 0, sizeof(sin));
	sin.sin_family = AF_INET;
	sin.sin_addr.s_addr = INADDR_ANY;

    /* Map service name to port number */

    if ( pse = getservbyname(service, protocol) )
        sin.sin_port = htons(ntohs((u_short)((u_short)pse->s_port + portbase)));
    else if ( (sin.sin_port = htons((u_short)atoi(service))) == 0 ) {
        printf("\nsockio: can't get %s service entry\n", service);
        return SYSERR;
    }

    /* Map protocol name to protocol number */

    if ( (ppe = getprotobyname(protocol)) == 0) {
        printf("\nsockio: can't get %s protocol entry\n", protocol);
        return SYSERR;
    }

    /* Use protocol to choose a socket type */

    if (strcmp(protocol, "udp") == 0)
        type = SOCK_DGRAM;
	else
        type = SOCK_STREAM;

    /* Allocate a socket */

    s = socket(PF_INET, type, ppe->p_proto);

    if (s < 0) {
        printf("\nsockio: can't create socket: error %d\n", h_errno);
        return SYSERR;
    }

    /* Bind the socket */

    if (bind(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
        closesocket(s);
        return SYSERR;
    }

    if (type == SOCK_STREAM && listen(s, qlen) < 0) {
        printf("\nsockio: can't listen on %s port: error %d\n", service, h_errno);
        return SYSERR;
    }

    /* Enable UDP broadcasts */

    if (type == SOCK_DGRAM)
        setsockopt(s, SOL_SOCKET, SO_BROADCAST, &option, sizeof(BOOL));

    return s;
}

/*------------------------------------------------------------------------
 * wname2ip - resolve a DNS name
 *------------------------------------------------------------------------
*/
int 
wname2ip(ip, nam)
IPaddr  ip;             /* resolved IP address */
char    *nam;           /* name to resolve */
{
    struct hostent  *phe;       /* pointer to host information entry */
    struct sockaddr_in sin;     /* an Internet endpoint address */

    memset((char *)&sin, 0, sizeof(sin));
	sin.sin_family = AF_INET;

    /* Map host name to IP address, allowing for dotted decimal */

    if ((sin.sin_addr.s_addr = inet_addr(nam)) == INADDR_NONE)
        if ( phe = gethostbyname(nam))
            memcpy(ip, phe->h_addr, phe->h_length);
        else
            return SYSERR;
    else {
        memcpy(ip, &sin.sin_addr.s_addr, 4);
        return OK;
    }
    return OK;
}

/*------------------------------------------------------------------------
 * wip2name - find the DNS name for a given IP address
 *------------------------------------------------------------------------
*/
int 
wip2name(nam, ip)
IPaddr  ip;             /* IP address to resolve */
char    *nam;           /* resolved name */
{
    struct hostent  *phe;   /* pointer to host information entry */

    if (phe=gethostbyaddr(ip, 4, AF_INET)) {
        strcpy(nam, phe->h_name);
        return OK;
    }
    return SYSERR;
}

/*------------------------------------------------------------------------
 *  wip2dot - create the dotted decimal form of ip in pdot
 *------------------------------------------------------------------------
 */
int  wip2dot(pdot, ip)
char	*pdot;
IPaddr	ip;
{
	char	*pch = pdot;
	int	i;

    sprintf(pch, "%u", ip[0] & 0xff);
	pch += strlen(pch);
	for (i=1; i<IP_ALEN; ++i) {
        sprintf(pch, ".%u", ip[i] & 0xff);
		pch += strlen(pch);
	}
    *pch = 0;
    return OK;
}

/*------------------------------------------------------------------------
 *  hl2net - convert long from host to net byte order
 *------------------------------------------------------------------------
 */
unsigned long  hl2net(unsigned long hlong)
{
    return htonl(hlong);
}

/*------------------------------------------------------------------------
 *  hs2net - convert short from host to net byte order
 *------------------------------------------------------------------------
 */
unsigned short  hs2net(unsigned short hshort)
{
    return htons(hshort);
}

/*------------------------------------------------------------------------
 *  net2hl - convert long from net to host byte order
 *------------------------------------------------------------------------
 */
unsigned long  net2hl(unsigned long hlong)
{
    return ntohl(hlong);
}

/*------------------------------------------------------------------------
 *  net2hs - convert short from net to host byte order
 *------------------------------------------------------------------------
 */
unsigned short  net2hs(unsigned short hshort)
{
    return ntohs(hshort);
}

/*------------------------------------------------------------------------
 *  getInfo - get socket info
 *------------------------------------------------------------------------
 */
int  getInfo(SOCKET s, char *buf)
{
    struct saddr info;
    int len = sizeof(struct saddr);
    unsigned short port;
    char tmp[16];

    if (getsockname(s, (struct sockaddr *) &info, &len) == SOCKET_ERROR) {
        strcpy(buf, "no info");
        return OK;
    }

    wip2dot(buf, info.sin_addr);
    strcat(buf, ":");
    port = net2hs(info.sin_port);
    sprintf(tmp, "%d", port);
    strcat(buf, tmp);

    return OK;
}

/*------------------------------------------------------------------------
 *  sCleanup  --  close all sockets for the specified PID
 *------------------------------------------------------------------------
*/

int 
sCleanup(int pid)
{
    return 0;
}

//
// exec - execute an application and don't wait for completion
//
int exec(char *cmdline, short flag)
{
    BOOL result;
    STARTUPINFO si;
    PROCESS_INFORMATION pi;

    GetStartupInfo(&si);

    si.dwFlags |= STARTF_USESHOWWINDOW;
    si.wShowWindow = flag;

    ZeroMemory(&pi, sizeof(PROCESS_INFORMATION));

    result = CreateProcess(
    (LPCTSTR)               NULL,
    (LPTSTR)                cmdline,
    (LPSECURITY_ATTRIBUTES) NULL,
    (LPSECURITY_ATTRIBUTES) NULL,
    (BOOL)                  FALSE,
    (DWORD)                 DETACHED_PROCESS | NORMAL_PRIORITY_CLASS,
    (LPVOID)                NULL,
    (LPCTSTR)               NULL,
    (LPSTARTUPINFO)         &si,
    (LPPROCESS_INFORMATION) &pi);

    if (result)
         return 0;

    return GetLastError();
}

